from sympy import BlockMatrix, GF, Matrix, eye, zeros
from sympy.polys.matrices import DomainMatrix

I = eye(5)
O = zeros(5, 5)

B = Matrix([
    [1, 1, 0, 0, 0],
    [1, 1, 1, 0, 0],
    [0, 1, 1, 1, 0],
    [0, 0, 1, 1, 1],
    [0, 0, 0, 1, 1],
])

A = BlockMatrix([
    [B, I, O, O, O],
    [I, B, I, O, O],
    [O, I, B, I, O],
    [O, O, I, B, I],
    [O, O, O, I, B],
]).as_explicit()

C = Matrix([
    [0, 1, 0, 0, 0],
    [1, 0, 1, 0, 0],
    [0, 1, 0, 1, 0],
    [0, 0, 1, 0, 1],
    [0, 0, 0, 1, 0],
])

A_neigh = BlockMatrix([
    [C, I, O, O, O],
    [I, C, I, O, O],
    [O, I, C, I, O],
    [O, O, I, C, I],
    [O, O, O, I, C],
]).as_explicit()

def rref_over_field(matrix, field):
    """Transform the given matrix to reduced row echelon form over the
    given field.
    """
    domain_matrix = DomainMatrix.from_Matrix(matrix).convert_to(field)
    domain_matrix_rref = domain_matrix.rref()[0]
    return domain_matrix_rref.to_Matrix()

def solve(matrix):
    """Given a matrix with dimensions 25x25, return the matrices E and R
    that are produced by augmenting the input matrix with the 25x25
    identity matrix, then transforming it to reduced row echelon form.
    """
    augmented_matrix = BlockMatrix([matrix, eye(25)]).as_explicit()
    ER = rref_over_field(augmented_matrix, GF(2))
    E = ER[:, 0:25]
    R = ER[:, 25:50]
    return E, R

E, R = solve(A)

print(rref_over_field(R, GF(2)))
